class PaymentProcessor {
  constructor() {
    if (this.constructor === PaymentProcessor) {
      throw new Error("Cannot instantiate abstract class");
    }
  }

  async createOrder() {
    throw new Error("Function createOrder() must be implemented");
  }

  async captureOrder() {
    throw new Error("Function captureOrder() must be implemented");
  }
  async refundOrder() {
    throw new Error("Function refundOrder() must be implemented");
  }

  /**
   * Turns class object into serializable data to store in redis db
   *
   * @returns {Object}
   */
  serialize() {
    throw new Error("Function serialize() must be implemented");
  }

  /**
   * Turns class object into serializable data to store in redis db
   *
   * @returns {PaymentProcessor}
   */
  static DESERIALIZE() {
    throw new Error("Function deserialize() must be implemented");
  }

  /**
   * Turns serialized data into class object of type PaymentProcessor
   *
   * @param {Object} serialized_data
   * @param {string} serialized_data.processor_name
   *
   * @returns {PaymentProcessor}
   */
  static LOAD_PAYMENT_PROCESSOR(serialized_data) {
    if (!("processor_name" in serialized_data)) {
      throw new Error(
        "Serialized data does not contain a hint to the payment processor in serialized_data.processor_name"
      );
    }

    let processors = {
      Paypal: require("./Paypal"),
    };

    if (!(serialized_data.processor_name in processors)) {
      throw new Error(
        `Cannot find Payment Processor ${serialized_data.processor_name}`
      );
    }

    return processors[serialized_data.processor_name].DESERIALIZE(
      serialized_data
    );
  }
}

module.exports = PaymentProcessor;
