const paypal_client = require("@paypal/paypal-js");

initialize_paypal_payments();

function initialize_paypal_payments() {
  let script_data = {
    "client-id": document.querySelector("input[name=paypal-client-id]").value,
    components: [
      "buttons",
      "marks",
      "payment-fields",
      "funding-eligibility",
      "hosted-fields",
    ],
    currency: "EUR",
  };

  let client_token_field = document.querySelector(
    "input[name=paypal-client-token]"
  );
  if (client_token_field) {
    script_data["data-client-token"] = client_token_field.value;
  }

  let funding_source = document.querySelector(
    "input[name=paypal-funding-source]"
  ).value;

  if (funding_source !== "paypal") {
    script_data["enable-funding"] = funding_source;
  }

  let checkout_data = get_paypal_checkout_data(funding_source);

  paypal_client
    .loadScript(script_data)
    .then((paypal) => {
      if (!paypal.isFundingEligible(funding_source)) {
        // Funding with this source is not available to the user
        let disabled_funding = localStorage.getItem("funding_not_eligible");
        if (disabled_funding === null) {
          disabled_funding = [];
        } else {
          disabled_funding = JSON.parse(disabled_funding);
        }
        disabled_funding.push(funding_source);
        localStorage.setItem(
          "funding_not_eligible",
          JSON.stringify(disabled_funding)
        );
        document.location.href = document.querySelector(
          "input[name=funding-source-not-eligible-url]"
        ).value;
        return;
      }
      if (funding_source === "card") {
        loadCardPayment(checkout_data);
      } else if (funding_source === "paypal") {
        paypal
          .Buttons(get_paypal_checkout_data(null))
          .render("#paypal-payment-button");
      } else {
        paypal
          .PaymentFields({
            fundingSource: funding_source,
            style: {},
            fields: {},
          })
          .render("#paypal-payment-fields");
        let button_data = get_paypal_checkout_data(funding_source);
        paypal.Buttons(button_data).render("#paypal-payment-button");
      }
    })
    .catch((err) => {
      // ToDo Handle error
      console.error("failed to load the PayPal JS SDK script", err);
    });
}

function get_paypal_checkout_data(funding_source) {
  let checkout_data = {
    style: {
      color: "white",
      height: 50,
    },
    fundingSource: funding_source,
    createOrder: (data, actions) => {
      let checkout_paypal_create_order_url =
        document.querySelector(
          "#paypal-checkout input[name=paypal-order-base-url]"
        ).value + "/create";

      return fetch(checkout_paypal_create_order_url, {
        method: "POST",
        headers: {
          "Content-Type": "application/json;charset=utf-8",
        },
      })
        .then((response) => response.json())
        .then((order) => {
          document.getElementById("paypal-checkout").dataset.order_id =
            order.order_id;
          return order.paypal_order_id;
        });
    },
    onCancel: () =>
      cancelPayment(
        document.getElementById("paypal-checkout").dataset.order_id
      ),
    onError: (err) => {
      console.error(err);
      return cancelPayment(
        document.getElementById("paypal-checkout").dataset.order_id
      );
    },
    onApprove: (data, actions) => {
      let checkout_paypal_capture_order_url =
        document.querySelector(
          "#paypal-checkout input[name=paypal-order-base-url]"
        ).value + "/capture";
      return fetch(checkout_paypal_capture_order_url, {
        method: "POST",
        headers: {
          "Content-Type": "application/json;charset=utf-8",
        },
        body: JSON.stringify({
          order_id: document.getElementById("paypal-checkout").dataset.order_id,
        }),
      })
        .then((response) => {
          if (response.status !== 200) {
            return response.json().then((json_response) => {
              throw json_response;
            });
          } else {
            return response.json();
          }
        })
        .then((orderData) => {
          if (typeof orderData.redirect_url !== "undefined") {
            document.location.href = orderData.redirect_url;
          }
        });
    },
    onInit: () => {
      document
        .getElementById("loading_paypal_funding_source")
        .classList.add("hidden");
      document
        .getElementById("paypal-payment-fields")
        .classList.remove("hidden");
      document
        .getElementById("paypal-payment-button")
        .classList.remove("hidden");
    },
  };
  if (funding_source) {
    checkout_data.fundingSource = funding_source;
  }
  return checkout_data;
}

function cancelPayment(order_id) {
  let checkout_paypal_cancel_order_url =
    document.querySelector("#paypal-checkout input[name=paypal-order-base-url]")
      .value + "/cancel";
  return fetch(checkout_paypal_cancel_order_url, {
    method: "POST",
    headers: {
      "Content-Type": "application/json;charset=utf-8",
    },
    body: JSON.stringify({
      order_id: order_id,
    }),
  });
}

function loadCardPayment(checkout_data) {
  // Show the Form
  document
    .getElementById("loading_paypal_funding_source")
    .classList.add("hidden");
  document.getElementById("paypal-payment-card").classList.remove("hidden");
  paypal.HostedFields.render({
    styles: {
      input: {
        padding: "0 .5rem",
        "font-family": "Liberation Sans",
      },
    },
    fields: {
      number: {
        selector: "#card-number",
        placeholder: "4111 1111 1111 1111",
      },
      expirationDate: {
        selector: "#expiration-date",
        placeholder: "MM/YY",
      },
    },
    createOrder: checkout_data.createOrder,
  }).then((cardFields) => {
    document
      .getElementById("paypal-card-form")
      .addEventListener("submit", (e) => {
        e.preventDefault();

        // Hide all errors
        document
          .querySelectorAll("#paypal-card-errors > p")
          .forEach((element) => {
            element.classList.add("hidden");
          });
        document
          .querySelectorAll("#paypal-card-form label")
          .forEach((element) => {
            element.classList.remove("error");
          });
        // Disable Button
        let submit_button = document.getElementById("submit-credit-card");
        submit_button.disabled = true;
        submit_button.classList.add("loading");

        let cardfield_data = {
          contingencies: ["SCA_ALWAYS"],
        };
        if (document.getElementById("card-holder-name")) {
          cardfield_data.cardholderName =
            document.getElementById("card-holder-name").value;
        }

        cardFields
          .submit(cardfield_data)
          .then((payload) => {
            checkout_data.onApprove().catch((reason) => {
              let payment_failed = false;
              reason.errors.forEach((error) => {
                if (error.type && error.type === "PAYPAL_CARD_3D_ERROR") {
                  payment_failed = true;
                  document
                    .getElementById("paypal-card-errors-3d")
                    .classList.remove("hidden");
                  cancelPayment(
                    document.getElementById("paypal-checkout").dataset.order_id
                  );
                } else if (
                  error.type &&
                  error.type === "PAYMENT_NOT_COMPLETED_ERROR"
                ) {
                  payment_failed = true;
                  document
                    .getElementById("paypal-card-errors-generic")
                    .classList.remove("hidden");
                  cancelPayment(
                    document.getElementById("paypal-checkout").dataset.order_id
                  );
                } else {
                  // Capture was not yet possible ToDo redirect user to Order
                  // There is a chance that webhooks will capture the payment
                  console.error(reason);
                  console.log("Fehlgeschlagen");
                }
              });
            });
            // Enable Button
            submit_button.disabled = undefined;
            submit_button.classList.remove("loading");
          })
          .catch((reason) => {
            reason.details.forEach((detail) => {
              cancelPayment(
                document.getElementById("paypal-checkout").dataset.order_id
              );
              if (detail.issue === "CARD_TYPE_NOT_SUPPORTED") {
                document
                  .getElementById("paypal-card-errors-invalid-card")
                  .classList.remove("hidden");
              } else if (detail.issue === "CARD_EXPIRED") {
                document
                  .getElementById("paypal-card-errors-expired")
                  .classList.remove("hidden");
              } else if (detail.field === "/payment_source/card/expiry") {
                document
                  .querySelector("#paypal-card-form label[for=expiration-date]")
                  .classList.add("error");
              } else if (detail.field === "/payment_source/card/number") {
                document
                  .querySelector("#paypal-card-form label[for=card-number]")
                  .classList.add("error");
              }
              // Enable Button
              submit_button.disabled = undefined;
              submit_button.classList.remove("loading");
            });
          });
      });
  });
}

module.exports = initialize_paypal_payments;
